import ac
import acsys
import time
import json
from pyKey import press
import threading
from os.path import dirname, exists, join
from os import mkdir
import ctypes
import configparser
from third_party.acdynamicbias_sim_info import SimInfoDynBias
simInfo = SimInfoDynBias()

appName = "ACDynamicBias"
width, height = 305 , 200


bb_prev = 0.0
distance = 0.0
dist = 0.0
timee = time.time()
record = False
car = simInfo.static.carModel
track = ac.getTrackName(0) + "_" + ac.getTrackConfiguration(0)
label = None
flag = False
start = False
config_path = None
bb_front = ""
bb_rear = ""
bb_zones = []


def get_documents_path():
    CSIDL_PERSONAL = 5  # My Documents
    SHGFP_TYPE_CURRENT = 0  # Get default value
    buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
    ctypes.windll.shell32.SHGetFolderPathW(None, CSIDL_PERSONAL, None, SHGFP_TYPE_CURRENT, buf)
    return buf.value


def get_buttons():
    global bb_front, bb_rear

    config = configparser.ConfigParser()
    config.read(get_documents_path() + "/Assetto Corsa/cfg/controls.ini")

    balance_up = config['BALANCEUP']['KEY'].lower()
    balance_down = config['BALANCEDN']['KEY'].lower()

    try:
        if ';' in balance_up:
            bb_front = balance_up.split(" ; ")[1]
        else:
            bb_front = balance_up
        if ';' in balance_up:
            bb_rear = balance_down.split(" ; ")[1]
        else:
            bb_rear = balance_down
    except:
        ac.log(appName + ": You have not set up the keyboard keys to control ERS delivery")
        exit()


def get_bb():
    return float(round(simInfo.physics.brakeBias*100, 2))


def get_distance():
    return float(round(simInfo.graphics.normalizedCarPosition*100, 4))


def check_folder():
    path = dirname(__file__) + "/data"
    if not exists(path):
        mkdir(path)
        ac.log(appName + ": Creating folder" + path)

    if not exists(path + "/" + track):
        mkdir(path + "/" + track)
        ac.log(appName + ": Creating folder" + path + "/" + track)

    if not exists(path + "/" + track + "/" + car):
        mkdir(path + "/" + track + "/" + car)
        ac.log(appName + ": Creating folder" + path + "/" + track + "/" + car)

    ac.log(appName + ": Using config on: " + path + "/" + track + "/" + car)
    return path + "/" + track + "/" + car


def reload_configs():
    global config_path, bb_zones

    bb_zones = []

    if config_path is not None:
        with open(config_path + "/config.txt", "r") as input_file:
            for i in input_file.readlines():
                brake_bias, dist = i.split("#")
                try:
                    bb_zones.append((float(brake_bias), float(dist)))
                except ValueError:
                    ac.log(appName + ": Error while converting " + brake_bias + " - " + dist + " to float")

    else:
        config_path = check_folder()
        reload_configs()


class Keys(threading.Thread):
    def __init__(self, brake_bias, label):
        threading.Thread.__init__(self)
        self.brake_bias = brake_bias
        self.label = label

    def run(self):
        global bb_front, bb_rear
        bb = get_bb()
        count = 0
        while count != 10:
            if self.brake_bias == bb:
                ac.setText(self.label, "Status: Started")
                break

            if self.brake_bias > bb:
                # + bb
                press(bb_front, 0.2)

            else:
                # - bb
                press(bb_rear, 0.2)

            count += 1
            bb = get_bb()
            time.sleep(.2)


def acMain(ac_version):
    global appWindow, label, bb_prev

    get_buttons()

    appWindow = ac.newApp(appName)
    ac.setTitle(appWindow, appName)
    ac.setSize(appWindow, width, height)

    btn_record = ac.addButton(appWindow, "Record")
    ac.setSize(btn_record, 150, 30)
    ac.setPosition(btn_record, 5, 50)
    ac.setFontAlignment(btn_record, "center")
    ac.addOnClickedListener(btn_record, recordListener)

    btn_stop_record = ac.addButton(appWindow, "Stop Recording")
    ac.setPosition(btn_stop_record, 155, 50)
    ac.setSize(btn_stop_record, 150, 30)
    ac.setFontAlignment(btn_stop_record, "center")
    ac.addOnClickedListener(btn_stop_record, stopRecordListener)

    btn_delete = ac.addButton(appWindow, "Delete")
    ac.setPosition(btn_delete, 5, 80)
    ac.setSize(btn_delete, 150, 30)
    ac.setFontAlignment(btn_delete, "center")
    ac.addOnClickedListener(btn_delete, deleteListener)

    btn_update = ac.addButton(appWindow, "Reload configs")
    ac.setPosition(btn_update, 155, 80)
    ac.setSize(btn_update, 150, 30)
    ac.setFontAlignment(btn_update, "center")
    ac.addOnClickedListener(btn_update, updateListener)

    btn_start = ac.addButton(appWindow, "Start/Stop")
    ac.setPosition(btn_start, 5, 110)
    ac.setSize(btn_start, 300, 30)
    ac.setFontAlignment(btn_start, "center")
    ac.addOnClickedListener(btn_start, startListener)

    label = ac.addLabel(appWindow, "Status: IDLE")
    ac.setPosition(label, 5, 150)

    return appName


def updateListener(a, b):
    global label, start, flag, record

    record = False
    start = False
    flag = False

    reload_configs()
    ac.setText(label, "Status: config updated")
    ac.log(appName + ": config updated")


def startListener(a, b):
    global start, label
    start = not start
    reload_configs()
    if start:
        ac.setText(label, "Status: Started")
        ac.log(appName + ": Started")

    else:
        ac.setText(label, "Status: Stopped")
        ac.log(appName + ": Stopped")


def deleteListener(a, b):
    global label, config_path, start, flag, record
    record = False
    start = False
    flag = False

    with open(config_path + "/config.txt", "w") as input_file:
        input_file.write("")
    ac.setText(label, "Status: config deleted")
    ac.log(appName + ": config deleted")


def stopRecordListener(a, b):
    global record, label
    record = False
    # TODO: reload configs
    reload_configs()
    ac.setText(label, "Status: recording stopped")
    ac.log(appName + ": recording stopped")


def recordListener(a, b):
    global record, label, bb_prev, start, flag, config_path

    bb_prev = get_bb()
    #delete_prev_config()
    # TODO: create file
    config_path = check_folder()
    with open(config_path + "/config.txt", "w") as input_file:
        input_file.write("")

    record = True
    start = False
    flag = False
    ac.setText(label, "Status: recording started")
    ac.log(appName + ": recording started")


def acUpdate(deltaT):
    global record, bb_prev, timee, car, track, label, flag, start, distance, config_path , bb_zones, dist

    bb = get_bb()
    distance = get_distance()

    if bb_prev != bb and record:
        bb_prev = bb
        timee = time.time()
        dist = distance
        flag = True
        ac.setText(label, "Status: Brake Balance set to " + str(bb) + "% at " + str(dist))


    else:
        if time.time() - timee >= 3 and record and flag:
            flag = False
            if config_path is not None:
                with open(config_path + "/config.txt", "a+") as input_file:
                    input_file.write(str(bb) + "#" + str(dist) + "\n")

            else:
                config_path = check_folder()
                with open(config_path + "/config.txt", "a+") as input_file:
                    input_file.write(str(bb) + "#" + str(dist) + "\n")

            ac.log(appName + ": Brake Bias saved")
            ac.setText(label, "Status: brake balance saved " + str(bb) + "% at " + str(dist))

    if start and not record and len(bb_zones) > 0:
        for brake_bias, distt in bb_zones:
            if distance-0.1 <= distt <= distance+0.1 and simInfo.physics.speedKmh > 0:
                Keys(brake_bias, label).start()
                ac.setText(label, "Status: moving brake balance to " + str(brake_bias) + "%")
